@@ -3,28 +3,30 @@
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: main_lde.c,v 1.25 1998/07/05 18:20:39 sdh Exp $
+ *  $Id: main_lde.c,v 1.42 2002/01/12 04:56:31 scottheavner Exp $
  */
 
+#if HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#if HAVE_GETOPT_H
 #include <getopt.h>
+#endif
 #include <signal.h>
 #include <time.h>
 
 #include <sys/stat.h>
 
 #include "lde.h"
-#include "ext2fs.h"
-#include "minix.h"
-#include "msdos_fs.h"
-#include "no_fs.h"
+#include "allfs.h"
 #include "recover.h"
 #include "tty_lde.h"
-#include "xiafs.h"
 
 #ifdef HAS_CURSES
 #  include "curses.h"
@@ -42,13 +44,16 @@ struct _main_opts {
   int search_off;
   int fs_type;
   int grep_mode;
-  int scrubxiafs;
   int dump_all;
   unsigned dump_start;
   unsigned dump_end;
   void (*dumper)(unsigned long nr);
   char *search_string;
   char *recover_file_name;
+  int log_to_file;
+  int blocksize;
+  int superscan;
+  int skiptableread;
 };
 
 struct _search_types {
@@ -61,8 +66,6 @@ struct _search_types {
 /* Initialize some global variables */
 char *program_name = "lde";
 char *device_name = NULL;
-char *text_names[] = { "autodetect", "no file system" , "minix", "xiafs", "ext2fs", "msdos" };
-
 char *inode_map=NULL;
 char *zone_map=NULL;
 char *bad_map=NULL;
@@ -73,13 +76,17 @@ unsigned char *zone_count = NULL;
 struct sbinfo sb2, *sb = &sb2;
 struct fs_constants *fsc = NULL;
 
+char *badblocks_directory = NULL;
+
 int CURR_DEVICE = 0;
 volatile struct _lde_flags lde_flags = 
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } ;
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } ;
 
 void (*lde_warn)(char *fmt, ...) = tty_warn;
 int  (*mgetch)(void) = tty_mgetch;
 
+struct _lde_typedata lde_typedata[] = LDE_ALLTYPES;
+
 
 /* Check if device is mounted, return 1 if is mounted else 0 */
 static int check_mount(char *device_name)
@@ -88,7 +95,7 @@ static int check_mount(char *device_name)
   char *mtab;
   struct stat statbuf;
 
-  fd = open("/etc/mtab",O_RDONLY);
+  fd = open("/etc/mtab",O_RDONLY|O_BINARY);
   fstat(fd, &statbuf);
 
   mtab = malloc(statbuf.st_size+1);
@@ -112,7 +119,7 @@ static int check_mount(char *device_name)
 }
 
 /* Define a handler for Interrupt signals: Ctrl-C */
-static void handle_sigint(void)
+static void handle_sigint(int a, void * b, void *c)
 {
   lde_flags.quit_now = 1;
 }
@@ -130,10 +137,10 @@ int check_root(void)
   return 0;
 }
   
-void read_tables(int fs_type)
+void read_tables(int fs_type, unsigned long blocksize)
 {
-  char super_block_buffer[2048];
-  sb->blocksize = 2048; /* Want to read in two blocks */
+  char super_block_buffer[17*2048];
+  sb->blocksize = 17*2048; /* Want to read in two blocks */
 
   /* Pull in first two blocks from the file system.  Xiafs info is
    * in the first block.  Minix, ext2fs is in second block (to leave
@@ -141,18 +148,21 @@ void read_tables(int fs_type)
    */
  
   nocache_read_block(0UL,super_block_buffer,sb->blocksize);
-  lde_warn("User requested %s filesystem. Checking device . . .",text_names[fs_type]);
-  if ( ((fs_type==AUTODETECT)&&(MINIX_test(super_block_buffer))) || (fs_type==MINIX) ) {
-    MINIX_init(super_block_buffer);
-  } else if ( ((fs_type==AUTODETECT)&&(EXT2_test(super_block_buffer))) || (fs_type==EXT2) ) {
-    EXT2_init(super_block_buffer);
-  } else if ( ((fs_type==AUTODETECT)&&(XIAFS_test(super_block_buffer))) || (fs_type==XIAFS) ) {
-    XIAFS_init(super_block_buffer);
-  } else if ( ((fs_type==AUTODETECT)&&(DOS_test(super_block_buffer))) || (fs_type==DOS) ) {
-    DOS_init(super_block_buffer);
+  lde_warn("User requested %s filesystem. Checking device . . .",lde_typedata[fs_type].name);
+
+  if ( fs_type == AUTODETECT ) {
+    for ( ++fs_type ; fs_type<LAST_FSTYPE; fs_type++) {
+      if (lde_typedata[fs_type].test(super_block_buffer,1)) {
+	break;
+      }
+    }
+  }
+
+  if ( fs_type < LAST_FSTYPE ) {
+    lde_typedata[fs_type].init(super_block_buffer);
   } else {
     lde_warn("No file system found on device");
-    NOFS_init(super_block_buffer);
+    NOFS_init(super_block_buffer, blocksize);
   }
 }
 
@@ -171,14 +181,15 @@ static void usage(void)
 
 static void long_usage(void)
 {
-  fprintf(stderr,"This is %s (version %s), Usage %s %s\n",program_name,VERSION,program_name,USAGE_STRING);
+  fprintf(stderr,"This is %s (version %s), Usage %s %s\n",program_name,LDE_VERSION,program_name,USAGE_STRING);
                 /* 12345678900123456789012345678901234567890123456789012345678901234567890123456789% */
   fprintf(stderr,"   -i {number}          Dump inode to stdout (-I all inodes after {number})\n"
 	         "   -b {number}          Dump block to stdout (-B all blocks after {number})\n"
                  "   -N {number}          Number of blocks to dump (using -I or -B option)\n"
                  "   -d {number}          Dump block's data to stdout (binary format)\n"
+                 "   -s {number}          Override blocksize\n"
                  "   -S {string}          Search disk for data (questionable usefulness, try grep)\n"
-                 "   -T {type}            Search disk for data. type={gz, tgz, script, {filename}}\n"
+                 "   -T {type}            Search disk for data, type={gz, tgz, script, {filename}}\n"
                  "   -L {number}          Search length (when using specified filename)\n"
                  "   -O {number}          Search offset (when using specified filename)\n"
                  "   -N {number}          Starting search block (defaults to first data zone)\n"
@@ -189,8 +200,7 @@ static void long_usage(void)
                  "                            (requires --ilookup or -i/-I)\n"
                  "   --all                Search entire disk (else just unused portions)\n"
                  "   --blanked-indirects  Work around indirects with useless info (Linux 2.0 bug)\n"
-                 "   -t fstype            Overide the autodetect.\n"
-                 "                            fstype = {no, minix, xiafs, ext2fs, msdos}\n"
+                 "   -t fstype            Overide the autodetect (-t help lists supported fs's)\n"
                  "   --help               Output this screen\n"
                  "   --paranoid           Open the device read only\n"
                  "   --append             Always append to recovery file (if file exists)\n"
@@ -198,6 +208,8 @@ static void long_usage(void)
                  "   --version            Print version information\n"
                  "   --write              Allow writes to the device\n"
                  "   --file name          Specify filename to save recovered inodes to\n"
+                 "   --superscan          Search device for superblocks, may be used with -t\n"
+                 "   --badblocks dir      Override badblocks using files in dir\n"
 	  );
   exit(0);
 }
@@ -205,7 +217,7 @@ static void long_usage(void)
 static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
 {
   int option_index = 0, i;
-  char c;
+  int c;
   static char gzip_tar_type[] = { 31, 138, 8, 0 };
   static char gzip_type[] = { 31, 139 };
   struct _search_types search_types[] = {
@@ -214,14 +226,15 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
       { "script", "#!/", 3, 0 },
       { "", NULL, 0, 0 }
   };
+#if HAVE_GETOPT_LONG
   struct option long_options[] =
     {
-      {"scrubxiafs", 0, 0, 0},
-      {"unscrubxiafs", 0, 0, 0},
       {"version", 0, 0, 'v'},
       {"help", 0, 0, 'h'},
       {"inode", 1, 0, 'i'},
       {"block", 1, 0, 'b'},
+      {"blocksize", 1, 0, 's'},
+      {"bs", 1, 0, 's'},
       {"limit", 1, 0, 'l'},
       {"all", 0, 0, 'a'},
       {"grep", 0, 0, 'g'},
@@ -238,8 +251,12 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
       {"recoverable",0,0,'#'},
       {"append",0,0,'%'},
       {"file",1,0,'f'},
+      {"logtofile",0,0,'F'},
+      {"superscan",0,0,'P'},
+      {"badblocks",0,0,'X'},
       {0, 0, 0, 0}
     };
+#endif
 
 
   /* if (argc && *argv)
@@ -248,28 +265,31 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
   while (1) {
     option_index = 0;
 
-    c = getopt_long (argc, argv, "avf:I:i:n:N:B:b:D:d:gpqS:t:T:whH?O:L:",
+#if HAVE_GETOPT_LONG
+    c = getopt_long (argc, argv, "avFf:I:i:n:N:B:b:D:d:gpPqs:S:t:T:X:whH?O:L:",
 		     long_options, &option_index);
+#else
+    c = getopt (argc, argv, "avFf:I:i:n:N:B:b:D:d:gpPqs:S:t:T:X:whH?O:L:");
+#endif
 
     if (c == -1)
       break;
 
     switch(c)
       {
-      case 0: /* Some XIAFS utils of limited usefulness */
+
+      case 0: /* long options only */
 	switch (option_index)
 	  {
 	  case 0:
-	     opts->scrubxiafs = 1;
+	  case 1:
 	     break;
-	   case 1:
-	     opts->scrubxiafs = -1;
-	     break;
-	   }
+	  }
+	break;
 
       case 'V': /* Display version */
       case 'v':
-	lde_warn("This is %s (version %s).",program_name,VERSION);
+	lde_warn("This is %s (version %s).",program_name,LDE_VERSION);
 	exit(0);
 	break;
       case 'a': /* Search disk space marked in use as well as unused */
@@ -307,17 +327,24 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
       case 'p': /* open FS read only */
 	lde_flags.paranoid = 1;
 	break;
+      case 'P': /* Superblock scanner */
+	opts->superscan = 1;
+	opts->skiptableread = 1;
+	break;
       case 'q': /* no audio -- well nop beeps */
 	lde_flags.quiet = 1;
 	break;
+      case 's': /* Override blocksize */
+	opts->blocksize = read_num(optarg);
+	break;
       case 'S': /* Search for a string of data */
 	opts->search_string = optarg;
 	opts->search_len = strlen(opts->search_string);
 	break;
       case 't': /* Specify the FS on the disk */
 	i = NONE;
-	while (text_names[i]) {
-	  if (!strncmp(optarg, text_names[i], strlen(optarg))) {
+	while (lde_typedata[i].name) {
+	  if (!strncmp(optarg, lde_typedata[i].name, strlen(optarg))) {
 	    opts->fs_type = i;
 	    break;
 	  }
@@ -327,8 +354,8 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
 	  lde_warn("`%s' type not recognized.",optarg);
 	  i = NONE;
 	  fprintf(stderr,"Supported file systems include: ");
-	  while (text_names[i]) {
-	    fprintf(stderr,"\"%s\" ",text_names[i]);
+	  while (lde_typedata[i].name) {
+	    fprintf(stderr,"\"%s\" ",lde_typedata[i].name);
 	    i++;
 	  }
 	  fprintf(stderr,"\n");
@@ -346,7 +373,7 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
 	}
 
 	if (opts->search_string==NULL) {
-	  i = open(optarg,O_RDONLY);
+	  i = open(optarg,O_RDONLY|O_BINARY);
 	  if (i) {
 	    opts->search_string = malloc(MAX_BLOCK_SIZE);
 	    if (read(i,opts->search_string,MAX_BLOCK_SIZE)<=0) {
@@ -359,8 +386,14 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
 	  }
 	}
 	break;
+      case 'X': /* Where we'll look for data on disk read errors */
+	badblocks_directory = optarg;
+	break;
       case 'O': /* Set offset for search string */
 	opts->search_off = read_num(optarg);
+	break;
+	  case 'F': /* Log all errors/messages to /tmp/ldeerrors */
+	lde_flags.logtofile = 1;
 	break;
       case 'L': /* Set length for search string */
 	opts->search_len = read_num(optarg);
@@ -374,6 +407,8 @@ static void parse_cmdline(int argc, char ** argv, struct _main_opts *opts)
 	break;
       case '!': /* Search for indirect blocks. */
 	lde_flags.indirect_search = 1;
+	opts->search_string = "";
+	opts->search_len = 0;
 	break;
       case '@': /* Lookup inodes on search matches. */
 	lde_flags.inode_lookup = 1;
@@ -419,12 +454,15 @@ int main(int argc, char ** argv)
   struct Generic_Inode *GInode = NULL;
 
   sigset_t sa_mask;
-  struct sigaction intaction = { (void *)handle_sigint, sa_mask, SA_RESTART, NULL };
+  struct sigaction intaction ;
 
-  struct _main_opts main_opts = { 0, 0, 0, AUTODETECT, 0, 0, 0, 0UL, 0UL, NULL, NULL, NULL };
+  struct _main_opts main_opts = { 0, 0, 0, AUTODETECT, 0, 0, 0UL, 0UL, NULL, NULL, NULL, 0, 0, 0, 0 };
 
   /* Set things up to handle control-c:  just sets lde_flags.quit_now to 1 */
   sigemptyset(&sa_mask);
+  intaction.sa_handler = (void *)handle_sigint;
+  intaction.sa_mask = sa_mask;
+  intaction.sa_flags = SA_RESTART;
   sigaction(SIGINT,&intaction,NULL);
 
   parse_cmdline(argc, argv, &main_opts);
@@ -434,18 +472,18 @@ int main(int argc, char ** argv)
 
 #ifndef PARANOID
   if (!lde_flags.paranoid) {
-    CURR_DEVICE = open(device_name,O_RDWR);
+    CURR_DEVICE = open(device_name,O_RDWR|O_BINARY);
     if (CURR_DEVICE < 0) {
       lde_warn("No write access to \"%s\",  attempting to open read-only.",
 	       device_name);
-      CURR_DEVICE = open(device_name,O_RDONLY);
+      CURR_DEVICE = open(device_name,O_RDONLY|O_BINARY);
       lde_flags.write_ok = 0;
     }
   } else
 #endif
   {
     lde_warn("Paranoid flag set.  Opening \"%s\" read-only.",device_name);
-    CURR_DEVICE = open(device_name,O_RDONLY);
+    CURR_DEVICE = open(device_name,O_RDONLY|O_BINARY);
   }
   
   if (CURR_DEVICE < 0) {
@@ -456,23 +494,24 @@ int main(int argc, char ** argv)
   for (i=0 ; i<3 ; i++)
     sync();
 
-  NOFS_init(NULL);
+  NOFS_init(NULL,0);
 
-  if (main_opts.scrubxiafs) {
-    XIAFS_scrub(main_opts.scrubxiafs);
-    exit(0);
+  if (main_opts.skiptableread==0) {
+    read_tables(main_opts.fs_type, main_opts.blocksize);
+
+  /* Override blocksize, if desired */
+  } else if (main_opts.blocksize) {
+    sb->blocksize = main_opts.blocksize;
   }
 
-  read_tables(main_opts.fs_type);
-
   /* Process requests handled by tty based lde */
   if (main_opts.recover_file_name!=NULL) {
     /* Check if file exists, if so, check if append flag is set and open accordingly */
-    if ( ( (fp = open(main_opts.recover_file_name,O_RDONLY)) > 0 ) && lde_flags.always_append ) {
+    if ( ( (fp = open(main_opts.recover_file_name,O_RDONLY|O_BINARY)) > 0 ) && lde_flags.always_append ) {
       close(fp);
-      fp = open(main_opts.recover_file_name,O_WRONLY|O_APPEND);
+      fp = open(main_opts.recover_file_name,O_WRONLY|O_APPEND|O_BINARY);
     } else {  /* It's ok to create a new file */
-      fp = open(main_opts.recover_file_name,O_WRONLY|O_CREAT,0644);
+      fp = open(main_opts.recover_file_name,O_WRONLY|O_CREAT|O_BINARY,0644);
     }
    
     /* Make sure we got a valid file number, if so look up inode and recover it, else print warning */
@@ -534,11 +573,11 @@ int main(int argc, char ** argv)
                 }
 		if (fsc->inode->i_size) {
 		  if (GInode->i_size>1024*1024)
-		    printf(" %5.0fM",(double)GInode->i_size/1024.0/1024.0);
+		    printf(" %5.0fMB",(double)GInode->i_size/1024.0/1024.0);
 		  else if (GInode->i_size>1024)
-		    printf(" %5.0fK",(double)GInode->i_size/1024.0);
+		    printf(" %5.0fKB",(double)GInode->i_size/1024.0);
 		  else
-		    printf(" %5.0fb",(double)GInode->i_size);
+		    printf(" %5.0fbytes",(double)GInode->i_size);
 		}
 		printf("\n");
 	    }
@@ -582,7 +621,8 @@ int main(int argc, char ** argv)
       }
       main_opts.dumper(nr);
     }
-    printf("\n"); /* Even up the command line for exit */
+    if (main_opts.dumper != ddump_block)
+       printf("\n"); /* Even up the command line for exit */
     exit(0);
   } else if (main_opts.grep_mode) {
     parse_grep();
@@ -590,6 +630,9 @@ int main(int argc, char ** argv)
   } else if (main_opts.search_string!=NULL) {
     search_fs(main_opts.search_string, main_opts.search_len, main_opts.search_off,main_opts.dump_end);
     exit(0);
+  } else if (main_opts.superscan) {
+    search_for_superblocks(main_opts.fs_type);
+    exit(0);
   }
 
 #ifdef HAS_CURSES
