@@ -1,9 +1,9 @@
-/*
+ /*
  *  lde/nc_block.c -- The Linux Disk Editor
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: nc_block.c,v 1.27 1998/08/11 07:22:27 sdh Exp $
+ *  $Id: nc_block.c,v 1.31 2001/11/26 00:07:23 scottheavner Exp $
  */
 
 #include <stdio.h>
@@ -106,7 +106,8 @@ static void cdump_block(unsigned long nr, bm_cursor *curs)
 /* Asks the user if it is ok to write the block to the disk, then goes
  * ahead and does it if the medium is writable.  The user has access to the
  * flags menu from this routine, to toggle the write flag */
-static int cwrite_block(cached_block *this_block, bm_cursor *curs, bm_flags *flags)
+static int cwrite_block(cached_block *this_block, bm_cursor *curs, bm_flags 
+*flags)
 {
   int c;
   char *warning = "(NOTE: write permission not set on disk,"
@@ -208,9 +209,9 @@ static void update_block_help(void)
   static char help_1[80], help_8[80];
 
   sprintf(help_1,"View block under cursor (%s block ptr is %d bytes)",
-	  text_names[fsc->FS],fsc->ZONE_ENTRY_SIZE);
+	  fsc->text_name,fsc->ZONE_ENTRY_SIZE);
   sprintf(help_8,"View inode under cursor (%s inode ptr is %d bytes)",
-	  text_names[fsc->FS],fsc->INODE_ENTRY_SIZE);
+	  fsc->text_name,fsc->INODE_ENTRY_SIZE);
   block_help[BHELP_BLOCK].description = help_1;
   block_help[BHELP_INODE].description = help_8;
 
@@ -382,7 +383,7 @@ static int update_block_data(cached_block **passed_block,
 int block_mode(void) {
   static unsigned char *copy_buffer = NULL;
   int icount = 0, val, v1 = 0, c = CMD_REFRESH, i;
-  char *HEX_PTR, *HEX_NOS = "0123456789ABCDEF";
+  char *HEX_PTR, *HEX_NOS = "0123456789ABCDEF", *s;
   unsigned long a, temp_ptr, search_iptr=0UL, inode_ptr[2] = { 0UL, 0UL };
   bm_flags flags = { 0, 0, 0, 0, 0, 1 };
   bm_cursor curs = { 0, 0, 0, 0, 0, 0, 16, VERT };
@@ -472,8 +473,10 @@ int block_mode(void) {
 	  curs.col = 0;
 	else if (curs.col>=curs.rs)
 	  curs.col = curs.rs - 1;
-	c = CMD_ANY_ACTION;                  /* Need a command that will fall through to refresh */
-	flags.dontwait = 1;                  /* Don't call lookup_key() on c, already has command */
+	c = CMD_ANY_ACTION;                  /* Need a command that will fall through 
+to refresh */
+	flags.dontwait = 1;                  /* Don't call lookup_key() on c, already 
+has command */
 	flags.highlight = 1;                 /* Update cursor position */
 	icount = 0;                          /* Abort any data entry in progress */
       }
@@ -536,7 +539,7 @@ int block_mode(void) {
 	break;
 
       case CMD_TOGGLE_ASCII: /* Toggle Ascii/Hex edit mode */
-	if (curs.rs != 16) { /* Should probably create a flag to indicate we
+	if (curs.rs == 16) { /* Should probably create a flag to indicate we
 			      * are in all ASCII MODE */
 	  flags.ascii_mode = 1 - flags.ascii_mode;
 	  flags.highlight = 1;
@@ -785,6 +788,27 @@ int block_mode(void) {
 	flags.redraw = flags.highlight = 1;
 	break;
 
+      case CMD_FIND_STRING: /* Search disk for data */
+        if (cwrite_block(this_block,&curs,&flags))
+	  break;
+	if ( ncread("Enter search string:",NULL,&s) > 0) {
+	  unsigned long mbnr;
+	  int moffset;
+	  if ( (search_blocks(s, current_block, &mbnr, &moffset)>0) ) {
+	    irecptr = NULL;
+	    clear_data_cache(this_block,&curs);
+	    update_block_data(&this_block,mbnr,irecptr,&curs,&flags,0);
+	    moffset -= curs.sow;
+	    curs.row = moffset / curs.rs;
+	    curs.col = moffset % curs.rs;
+	    flags.edit_block = 0;
+	    flags.redraw = 1;
+	    search_iptr = 0L;
+	  }
+
+	}
+	break;
+
       case CMD_FIND_INODE:
       case CMD_FIND_INODE_MC: /* Find an inode which references this block */
         lde_warn("Searching for inode containing block 0x%lX . . .",
@@ -921,8 +945,8 @@ int block_mode(void) {
       case CMD_NUMERIC_REF: /* Jump to a block by numeric reference */
         if (cwrite_block(this_block,&curs,&flags))
 	  break;
-        if (cread_num("Enter block number (leading 0x or $ indicates hex):"
-		      ,&a)) {
+        if (ncread("Enter block number (leading 0x or $ indicates hex):"
+		      ,&a,NULL)) {
 	  if (a >= sb->nzones)
 	    a=sb->nzones-1;
 	  irecptr = NULL;
@@ -985,9 +1009,3 @@ int block_mode(void) {
   return 0;
 }
 
-
-
-
-
-
-
