@@ -1,16 +1,20 @@
-/*
+ /*
  *  lde/nc_lde.c -- The Linux Disk Editor
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: nc_lde.c,v 1.32 1998/08/11 03:58:20 sdh Exp $
+ *  $Id: nc_lde.c,v 1.40 2002/01/11 18:30:41 scottheavner Exp $
  */
 
 #include <stdio.h>
-#include <strings.h>
+#include <string.h>
 #include <ctype.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#if HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
 #include <stdarg.h>
 
 #include "lde.h"
@@ -107,9 +111,11 @@ int cquery(char *data_string, char *data_options, char *warn_string)
  * what the hell.  If coutput begins with a space, cinput is returned in
  * coutput and the number processing does not occurr.
  */
-int cread_num(char *coutput, unsigned long *a)
+int ncread(char *coutput, unsigned long *a, char **string)
 {
-  char cinput[80];
+  static char cinput[80];
+
+  cinput[0] = 0;
 
   lde_flags.quit_now = 0;
 
@@ -139,13 +145,13 @@ int cread_num(char *coutput, unsigned long *a)
     delwin(window_available);
 # endif
 
-  if ((coutput[0]!=' ')&&(a)) {
+  if (!string) {
     *a = read_num(cinput);
     return strlen(cinput);
   } else {
     if (lde_flags.quit_now)  /* Control-C aborts */
       return -1;
-    strncpy(coutput, cinput, 80);
+	*string = cinput;
     return strlen(cinput);
   }
 
@@ -212,7 +218,7 @@ void restore_header(void)
 
   werase(header);
   sprintf(echo_string,"%s v%s : %s : %s",
-	  program_name,VERSION,text_names[fsc->FS],device_name);
+	  program_name,LDE_VERSION,fsc->text_name,device_name);
   mvwprintw(header,0,(COLS-strlen(echo_string))/2-1,echo_string);
   update_header();
 #endif
@@ -722,11 +728,10 @@ void crecover_file(unsigned long inode_zones[], unsigned long filesize)
 {
   static char recover_file_name[80] = "RECOVERED.file";
   int fp;
-  char recover_query[80];
+  char *recover_query;
   int c;
 
-  sprintf(recover_query," Write data to file:");
-  c = cread_num(recover_query, NULL);
+  c = ncread("Write data to file:", NULL, &recover_query);
   if (c<0) {
     lde_warn("Recovery aborted");
     return;
@@ -734,25 +739,25 @@ void crecover_file(unsigned long inode_zones[], unsigned long filesize)
     strncpy(recover_file_name, recover_query, 80);
   }
 
-  if ( (fp = open(recover_file_name,O_RDONLY)) > 0 ) {
+  if ( (fp = open(recover_file_name,O_RDONLY|O_BINARY)) > 0 ) {
     close(fp);
     fp = 0;
     if (lde_flags.always_append) {
-      fp = open(recover_file_name,O_WRONLY|O_APPEND);
+      fp = open(recover_file_name,O_WRONLY|O_APPEND|O_BINARY);
     } else {
       switch (cquery("File exists, append data [Yes/Overwrite/Quit]: ","yoq","")) {
         case 'y':
-	  fp = open(recover_file_name,O_WRONLY|O_APPEND);
+	  fp = open(recover_file_name,O_WRONLY|O_APPEND|O_BINARY);
 	  break;
         case 'o':
-	  fp = open(recover_file_name,O_WRONLY|O_TRUNC);
+	  fp = open(recover_file_name,O_WRONLY|O_TRUNC|O_BINARY);
 	  break;
         default:
 	  return;
 	  break;
       }
     }
-  } else if ( (fp = open(recover_file_name,O_WRONLY|O_CREAT,0644)) < 0 )
+  } else if ( (fp = open(recover_file_name,O_WRONLY|O_CREAT|O_BINARY,0644)) < 0 )
     lde_warn("Cannot open file '%s'",recover_file_name);
 
   if (fp > 0) {
@@ -812,15 +817,15 @@ int recover_mode(void)
       case REC_FILE12:
       case REC_FILE13:
       case REC_FILE14:
-	if (cread_num("Enter block number (leading 0x or $ indicates hex):", &a))
+	if (ncread("Enter block number (leading 0x or $ indicates hex):", &a, NULL))
 	  fake_inode_zones[c-REC_FILE0] = (unsigned long) a;
         break;
       case REC_FILE_SIZE:
-	if (cread_num("Enter file size (0 is special) (leading 0x or $ indicates hex):", &a))
+	if (ncread("Enter file size (0 is special) (leading 0x or $ indicates hex):", &a, NULL))
 	  fake_inode_zones[INODE_BLKS] = (unsigned long) a;
         break;
       case CMD_CLR_RECOVER:
-	bzero(fake_inode_zones,sizeof(long)*(INODE_BLKS+1));
+	bzero((char *)fake_inode_zones,sizeof(long)*(INODE_BLKS+1));
 	break;
       case CMD_BLOCK_MODE:
       case CMD_VIEW_SUPER:
@@ -1027,6 +1032,7 @@ void interactive_main(void)
       printf("* Your shell's TERM variable is unset.\n");
     }
     printf("* Try something like 'export TERM=vt100' or 'setenv TERM vt100'\n");
+    printf("* On older distros, you might try 'export TERMPATH=/usr/lib/terminfo' first.\n");
     return;
   }
   cbreak();
@@ -1073,7 +1079,7 @@ void interactive_main(void)
 # endif
 
   /* Clear out restore buffer */
-  bzero(fake_inode_zones,sizeof(long)*(INODE_BLKS+1));
+  bzero((void *)fake_inode_zones,sizeof(long)*(INODE_BLKS+1));
  
   /* Our three curses windows */
 # if HEADER_SIZE>0
