@@ -3,22 +3,25 @@
  *
  *  Copyright (C) 1994  Scott D. Heavner
  *
- *  $Id: recover.c,v 1.26 1998/06/21 06:57:16 sdh Exp $
+ *  $Id: recover.c,v 1.36 2002/01/12 05:42:53 scottheavner Exp $
  */
 
 #include <stdio.h>
 #include <stdlib.h>
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <ctype.h>
+#include <string.h>
 
 #ifdef   HAVE_UNAME
 #include <sys/utsname.h>
 #endif
-#include <linux/fs.h>
 
 #include "lde.h"
 #include "recover.h"
 #include "tty_lde.h"
+#include "allfs.h"
 
 /* This takes care of the mapping from a char pointer to unsigned
  * long/short, depending on the file system */
@@ -322,10 +325,10 @@ int advance_zone_pointer(unsigned long zone_index[], unsigned long *currblk,
   long sincrement,i;
 
   /* Check that we are at a block that is indexed by this inode */
-  if ((map_block(zone_index, *ipointer, &blknr))||(blknr!=(*currblk))) {
+  if ((FS_cmd.map_block(zone_index, *ipointer, &blknr))||(blknr!=(*currblk))) {
     /* We aren't: look it up using brute force */
     for (;;) {
-      result = map_block(zone_index, local_ipointer, &blknr);
+      result = FS_cmd.map_block(zone_index, local_ipointer, &blknr);
       if (result < EMB_HALT) {
 	return AZP_BAD_START;
       } else if (result < EMB_SKIP) {
@@ -347,7 +350,7 @@ int advance_zone_pointer(unsigned long zone_index[], unsigned long *currblk,
   for (i=0; i<increment; i++) {
     for (;;) {
       local_ipointer += sincrement;
-      result = map_block(zone_index, local_ipointer, &blknr);
+      result = FS_cmd.map_block(zone_index, local_ipointer, &blknr);
       if (result < EMB_HALT) {
 	return result;
       } else if (result < EMB_SKIP) {
@@ -415,9 +418,9 @@ int recover_file(int fp,unsigned long zone_index[],unsigned long filesize)
     if (lde_flags.blanked_indirects) 
       result = hacked_map_block(zone_index,j,&nr,&skipped);
     else 
-      result=map_block(zone_index,j,&nr);
+      result=FS_cmd.map_block(zone_index,j,&nr);
 #else
-    result=map_block(zone_index,j,&nr);
+    result=FS_cmd.map_block(zone_index,j,&nr);
 #endif
 
     /* Block successfully looked up? */
@@ -435,8 +438,13 @@ int recover_file(int fp,unsigned long zone_index[],unsigned long filesize)
 
     /* Have a valid block number? */
     if (nr) {
+      int k = 0;
+#ifdef MSDOSHACK /* Also added k in line above and below */
+#warning MSDOS HACK ENABLED
+      for ( k=0; k<sb->zonesize; k++) { /* MSDOS HACK */
+#endif
       /* Read data from disk */
-      dind = cache_read_block(nr,NULL,CACHEABLE);
+      dind = cache_read_block(nr+k,NULL,CACHEABLE);
       /* Check for small block if we opened a file instead of a device */
       write_count = (size_t) lookup_blocksize(nr);
       /* Add what we're about to write to total written */
@@ -452,6 +460,9 @@ int recover_file(int fp,unsigned long zone_index[],unsigned long filesize)
       /* Quit when we've written enough */
       if ((filesize)&&(written>=filesize))
 	return 0;
+#ifdef MSDOSHACK
+      } /* END MSDOS HACK */
+#endif
     }
     j++;
   }
@@ -480,7 +491,7 @@ int check_recover_file(unsigned long zone_index[],unsigned long filesize)
       return 1;
     }
 
-    result=map_block(zone_index,j,&nr);
+    result=FS_cmd.map_block(zone_index,j,&nr);
 
     if (result) {
       if (result < EMB_HALT) {
@@ -496,7 +507,7 @@ int check_recover_file(unsigned long zone_index[],unsigned long filesize)
     }
 
     if ((nr)&&FS_cmd.zone_in_use(nr)) {
-      lde_warn("Block %ld (0x%lX) in use by another file. Hit any key to continue (q=abort, l=lookup inode).",nr,nr);
+      lde_warn("Block %ld (0x%lX) in use. (q=abort, l=lkupinode, c=cont)",nr,nr);
       result = mgetch();
       if (result == 'q') {
         lde_warn("Check aborted.");
@@ -519,6 +530,7 @@ int check_recover_file(unsigned long zone_index[],unsigned long filesize)
 	}
       }
       cr_result = 1;
+      break;
     }
     j++;
   }
@@ -551,7 +563,7 @@ unsigned long find_inode(unsigned long nr, unsigned long last_nr)
       while (1) {
 	if (lde_flags.quit_now)
 	  return 0L;
-	if ((result=map_block(GInode->i_zone, b++, &test_block))) {
+	if ((result=FS_cmd.map_block(GInode->i_zone, b++, &test_block))) {
 	  if (result < EMB_HALT ) {
 	    break;
 	  } else if (result < EMB_SKIP ) {
@@ -656,21 +668,21 @@ void search_fs(unsigned char *search_string, int search_len, int search_off, uns
 	} while (++i<search_len);
 	
 	if (matched) {
-	  printf("Match at block 0x%lX",nr);
+	  fprintf(stderr,"Match at block 0x%lX",nr);
 	  if (lde_flags.inode_lookup) {
 	    if ( (inode_nr = find_inode(nr, 0UL)) ) {
-	      printf(", check inode 0x%lX",inode_nr);
+	      fprintf(stderr,", check inode 0x%lX",inode_nr);
 	      if (lde_flags.check_recover) {
 		lde_warn = no_warn;  /* Suppress output */
-		GInode = FS_cmd.read_inode(nr);
-		printf(", recovery %spossible",(check_recover_file(GInode->i_zone,GInode->i_size)?"":"NOT ") );
+		GInode = FS_cmd.read_inode(inode_nr);
+		fprintf(stderr,", recovery %spossible",((check_recover_file(GInode->i_zone,GInode->i_size)==0)?"":"NOT ") );
 		lde_warn = tty_warn; /* Reinstate output */
 	      }
 	    } else {
-	      printf(", no %sinode found",((lde_flags.search_all)?"":"unused ") );
+	      fprintf(stderr,", no %sinode found",((lde_flags.search_all)?"":"unused ") );
 	    }
 	  }
-	  printf(".\n");
+	  fprintf(stderr,".\n");
 	}
       }
 
@@ -691,7 +703,7 @@ void search_fs(unsigned char *search_string, int search_len, int search_off, uns
 	match_count[0] = match_count[1] = match_count[2] = match_count[3] =
 	  match_count[4] = largest_match = 0;
 	match[0] = dind[1];
-	for (i=3;i<BLOCK_SIZE/4;i+=fsc->ZONE_ENTRY_SIZE) {
+	for (i=(fsc->ZONE_ENTRY_SIZE-1);i<sb->blocksize/4;i+=fsc->ZONE_ENTRY_SIZE) {
 	  no_match_flag = 1;
 	  for (j=-1;j++<largest_match;)
 	    if (dind[i]==match[j]) {
@@ -707,30 +719,30 @@ void search_fs(unsigned char *search_string, int search_len, int search_off, uns
 	}
 	
 	i = 0;
-	for (j=0;j<largest_match;j++)
+	for (j=-1;j++<largest_match;)
 	  i+=match_count[j];
 	
-	/* A perfect match is 128, so maybe 100 is a little outrageous */
-	if (i>HB_COUNT) {
+	/* If we get a hit rate greater than 65%, lets run with it */
+	if (i>sb->blocksize/fsc->ZONE_ENTRY_SIZE/4*65/100) {
 	  /* The next block of code will search the low bytes for sequences.
 	   * Hopefully, a fair number of the low bytes will be in order.
 	   */ 
 	  match[0] = dind[0]; /* number in sequence 0,1,2,etc */
-	  match[1] = 0;       /* Length of current sequence */
-	  match[2] = 0;       /* Number of sequences longer than 3  */
-	  for (i=2;i<BLOCK_SIZE/4;i+=fsc->ZONE_ENTRY_SIZE) {
+	  match_count[1] = 0; /* Length of current sequence */
+	  match_count[2] = 0; /* Number of sequences longer than 3  */
+	  for (i=0;i<sb->blocksize/4;i+=fsc->ZONE_ENTRY_SIZE) {
 	    if (dind[i]==(++match[0]))
-	      match[1]++;
+	      match_count[1]++;
 	    else {
-	      if (match[1]>3) match[2]+=match[1];
-	      match[1] = 0;
+	      if (match_count[1]>3) match_count[2]+=match_count[1];
+	      match_count[1] = 0;
 	      match[0] = dind[i];
 	    }
 	  }
 	  
-	  if (match[2]>SEQ_COUNT) {
+	  if (match_count[2]>SEQ_COUNT) {
 	    fprintf(stderr,"%d hits on possible indirect block at 0x%lX\n",
-		    match[2],nr);
+		    match_count[2],nr);
 	    match_total++;
 	  }
 	}      
@@ -744,5 +756,116 @@ void search_fs(unsigned char *search_string, int search_len, int search_off, uns
 
 }
 
+/* Do a regular expression search across the filesystem */
+int search_blocks(char *searchstring, unsigned long sbnr, unsigned long *mbnr, int *moffset) {
+  int     retval=0;
+
+#if HAVE_MEMMEM
+  int     rs = 500;
+  size_t  bytesread, rbs = 0, lsearchstring;
+  char    *buffer = 0, *match;
+
+  if ( (!searchstring) || (!*searchstring) ) {
+    lde_warn("NULL searchstring ignored");
+    return -1;
+  }
+  
+  if (rs>sb->nzones) {
+    rs = sb->nzones - sbnr;
+  }
+  
+  /* Try to read 501 blocks at a time */
+  buffer = malloc((rs+1)*sb->blocksize);
+  if (!buffer) {
+    /* too ambitious? */
+    rs = 100;
+    buffer = malloc((rs+1)*sb->blocksize);
+  }
+  if (!buffer) {
+    /* still too ambitious? */
+    rs = 1;
+    buffer = malloc((rs+1)*sb->blocksize);
+  }
+  if (!buffer) {
+    lde_warn("Out of memory, can't complete search");
+    return -1;
+  }
+  
+  rbs = (rs+1)*sb->blocksize;
+  lsearchstring=strlen(searchstring);
+  
+  for ( ; sbnr<sb->nzones; sbnr+=rs) {
+    bytesread = nocache_read_block(sbnr, buffer, rbs);
+    if (bytesread < 0) {
+      retval = -1;
+      break;
+    }
+    if ( (match = memmem(buffer, bytesread, searchstring, lsearchstring)) ) {
+      *mbnr    = ((int)(match - buffer))/sb->blocksize + sbnr;
+      *moffset = ((int)(match - buffer))%sb->blocksize;
+      lde_warn("matches block=0x%lx offset=0x%lx",*mbnr,*moffset+sbnr*sb->blocksize);
+      retval   = 1;
+      break;
+    }
+    if (lde_flags.quit_now) {
+      lde_warn("Search aborted at block 0x%lx",sbnr);
+      retval = -2;
+      break;
+    }
+  }
+  
+  free(buffer);
+#else
+#warning No memmem() found, block search disabled.
+  lde_warn("Block search needs memmem, try recompiling.");
+#endif
+
+  if (retval==0)
+     lde_warn("No match");
+  
+  return retval;
+}
 
 
+int search_for_superblocks(int fs_type) {
+  unsigned long sbnr = 0;
+  int i;
+  char *buffer[512*500];
+  size_t  bytesread;
+ 
+  /* Want to do all searches on 512 byte boundries, in case something has gotten screwed up */
+  NOFS_init(NULL,512);
+
+  if ( fs_type >= LAST_FSTYPE || fs_type < AUTODETECT ) {
+    lde_warn("Bad filetype specified (i.e. why would \"no\" have a superblock) . . . Aborting.");
+    return -1;
+  }
+
+  lde_warn("Searching disk for %s superblocks . . .",(fs_type==AUTODETECT)?"any":lde_typedata[fs_type].name);
+
+  for ( ; sbnr<sb->nzones; ++sbnr) {
+    bytesread = nocache_read_block(sbnr, buffer, 512);
+    if (bytesread < 0)
+      break;
+
+    if ( fs_type == AUTODETECT ) {
+      for ( i = AUTODETECT+1 ; i<LAST_FSTYPE; i++) {
+	if (lde_typedata[i].test(buffer,0)) {
+	  lde_warn("Found %s superblock at 0x%lx",lde_typedata[i].name,sbnr);
+	}
+      }
+    } else {
+      if (lde_typedata[fs_type].test(buffer,0)) {
+	lde_warn("Found %s superblock at 0x%lx",lde_typedata[fs_type].name,sbnr);
+      }
+    }
+    
+    if (lde_flags.quit_now) {
+      lde_warn("Search aborted");
+      return -1;
+    }
+  }
+
+  lde_warn("Search complete");
+  return 0;
+}
